@@ -2,6 +2,10 @@
  *  Copyright (C) 2002,2003,2004  Mattia Dongili<dongili@supereva.it>
  *                                George Staikos <staikos@0wned.org>
  *
+ *  2004.08.22
+ *  - added percentage/absolute frequency translation based on a patch submitted
+ *    by Herv Eychenne
+ *
  *  2003.16.08
  *  - added support for cpu monitoring, base code by Dietz Proepper and minor
  *    fixes by Mattia Dongili
@@ -73,7 +77,7 @@ int parse_config (general *g) {
 
       profile *prof = (profile *)malloc(sizeof(profile));
       memset(prof, 0, sizeof(profile));
-      if (parse_config_profile (config, prof, g->has_sysfs) < 0) {
+      if (parse_config_profile (config, prof, g) < 0) {
         cp_log(LOG_CRIT, "parse_config(): [Profile] error parsing %s, see logs for details.\n", g->config_file);
         fclose(config);
         return -1;
@@ -110,7 +114,7 @@ int parse_config (general *g) {
       } else {
         r_iter = g->rules;
         /* check duplicate names */
-        while (r_iter) {
+        while (r_iter != NULL) {
           if (strcmp(r_iter->name, rul->name) == 0) {
             cp_log(LOG_CRIT, "parse_config(): [Rule] name \"%s\" already exists.\n", rul->name);
             return -1;
@@ -126,18 +130,26 @@ int parse_config (general *g) {
   }
   fclose(config);
 
+  /* did I read something? 
+   * chek if I read at least one rule, otherwise exit
+   */
+  if (g->rules == NULL) {
+    cp_log(LOG_ERR, "parse_config(): No rules found!\n");
+    return -1;
+  }
+  
   /*
    * associate rules->profiles
    * go through rules and associate to the proper profile
    */
-    r_iter = g->rules;
-    while (r_iter != NULL) {
-      int profile_found = 0;
+  r_iter = g->rules;
+  while (r_iter != NULL) {
+    int profile_found = 0;
 
-      p_iter = g->profiles;
-      /* go through profiles */
-      while (p_iter != NULL) {
-        if (strcmp(r_iter->profile_name, p_iter->name)==0) {
+    p_iter = g->profiles;
+    /* go through profiles */
+    while (p_iter != NULL) {
+      if (strcmp(r_iter->profile_name, p_iter->name)==0) {
         /* a profile is allowed to be addressed by more than 1 rule */
         r_iter->prof = p_iter;
         profile_found = 1;
@@ -253,7 +265,7 @@ char buf[256];
  *
  * Returns -1 if required properties are missing, 0 otherwise
  */
-int parse_config_profile (FILE *config, profile *p, const unsigned int has_sysfs) {
+int parse_config_profile (FILE *config, profile *p, general *g) {
 #define HAS_NAME    1
 #define HAS_MIN     2
 #define HAS_MAX     4
@@ -292,11 +304,6 @@ char buf[256];
       state |= HAS_NAME;
     } else if (strcmp(name,"minfreq")==0) {
       if (strstr(value, "%") != NULL) {
-        if (has_sysfs) {
-          cp_log(LOG_CRIT, "parse_config_profile(): minfreq=%s but using sysfs which doesn't \
-support percentage frequencies, please edit the configuration file and use absolute values instead.\n", value);
-          return -1;
-        }
         /* set separator (useful for 2.4 kernels only) */
         if (p->sep != ':') {
           p->sep = '%';
@@ -315,13 +322,10 @@ support percentage frequencies, please edit the configuration file and use absol
       }
       p->min_freq = atol(value);
       state |= HAS_MIN;
+      
     } else if (strcmp(name,"maxfreq") == 0) {
+      
       if (strstr(value, "%") != NULL) {
-        if (has_sysfs) {
-          cp_log(LOG_CRIT, "parse_config_profile(): maxfreq=%s but using sysfs which doesn't \
-support percentage frequencies, please edit the configuration file and use absolute values instead.\n", value);
-          return -1;
-        }
         /* set separator (useful for 2.4 kernels only) */
         if (p->sep != ':') {
           p->sep = '%';
@@ -340,10 +344,13 @@ support percentage frequencies, please edit the configuration file and use absol
       }
       p->max_freq = atol(value);
       state |= HAS_MAX;
+    
     } else if (strcmp(name,"policy") == 0) {
+      
       strncpy(p->policy_name, value, 255);
       p->policy_name[254] = 0;
       state |= HAS_POLICY;
+    
     } else {
       cp_log(LOG_WARNING, "WARNING: [Profile] skipping unknown config option \"%s\"\n", name);
     }
@@ -365,6 +372,27 @@ support percentage frequencies, please edit the configuration file and use absol
     cp_log(LOG_ERR, "parse_config_profile(): [Profile] \"%s\" missing required property \"policy\".\n", p->name);
     return -1;
   }
+  
+  /* translate max_freq to absolute value if necessary */
+  if (g->has_sysfs && p->sep == '%') {
+    p->max_freq = g->cpu_max_freq * ((float)p->max_freq / 100);
+    cp_log(LOG_INFO, "parse_config_profile(): [Profile] \"%s\" translated max frequency to %d\n", p->name, p->max_freq);
+    /* normalize if necessary */
+    if (p->max_freq > g->cpu_max_freq) {
+      cp_log(LOG_NOTICE, "parse_config_profile(): [Profile] \"%s\" max frequency %d out of range\n", p->name, p->max_freq);
+      p->max_freq = g->cpu_max_freq;
+    }
+  }
+  /* translate min_freq to absolute value if necessary */
+  if (g->has_sysfs && p->sep == '%') {
+    p->min_freq = g->cpu_max_freq * ((float)p->min_freq / 100);
+    cp_log(LOG_INFO, "parse_config_profile(): [Profile] \"%s\" translated min frequency to %d\n", p->name, p->min_freq);
+    /* normalize if necessary */
+    if (p->min_freq < g->cpu_min_freq) {
+      cp_log(LOG_NOTICE, "parse_config_profile(): [Profile] \"%s\" min frequency %d out of range\n", p->name, p->min_freq);
+      p->min_freq = g->cpu_min_freq;
+    }
+  }
 
   return 0;
 }
@@ -392,6 +420,9 @@ char buf[256];
   r->cpu = (cpu_interval *) malloc(sizeof(cpu_interval));
   r->cpu->min = 101;    /* so we won't ever trigger them... */
   r->cpu->max = -1;
+
+  /* initialize cpu_nice_scale to DEFAULT_NICE_SCALE */
+  r->cpu_nice_scale = DEFAULT_NICE_SCALE;
   
   while (!feof(config)) {
     char *clean;
@@ -430,6 +461,18 @@ char buf[256];
     } else if (strcmp(name,"cpu_interval") == 0) {
       r->cpu->min = atoi(strtok(value,"-"));
       r->cpu->max = atoi(strtok(NULL,""));
+    } else if (strcmp(name,"cpu_nice_scale") == 0) {
+      r->cpu_nice_scale = atof(value);
+      if (r->cpu_nice_scale<=0) {
+        cp_log(LOG_ERR, "parse_config_rule(): [Rule] invalid \"cpu_nice_scale\" value: %f.\n", r->cpu_nice_scale);
+        return -1;
+      }
+    } else if (strcmp(name,"delay_cycles") == 0) {
+      r->delay_cycles = atoi(value);
+      if (r->delay_cycles<0) {
+        cp_log(LOG_ERR, "parse_config_rule(): [Rule] invalid \"delay_cycles\" value: %d.\n", r->delay_cycles);
+        return -1;
+      }
     } else if (strcmp(name,"programs") == 0) {
       /* create program list */
       char *t_prog;
@@ -446,7 +489,7 @@ char buf[256];
         temp->string[254] = 0;
 
         string_list_append(r->program_list, temp);
-        /* cp_log(LOG_DEBUG, "parse_config_rule(): read program: %s\n", t_prog); */
+        cp_log(LOG_DEBUG, "parse_config_rule(): read program: %s\n", temp->string);
       } while ((t_prog = strtok(NULL,",")) != NULL);
 
     } else if (strcmp(name, "profile") == 0) {
@@ -569,6 +612,8 @@ void free_config(general *g) {
   g->poll_interval = DEFAULT_POLL;
   g->has_sysfs = 0;
   g->acpi_workaround = 0;
+  g->cpu_min_freq = 0;
+  g->cpu_max_freq = 0;
   
   if (!g->log_level_overridden)
     g->log_level = DEFAULT_VERBOSITY;
