@@ -19,7 +19,7 @@
 
 /* Configuration functions */
 int parse_config (general *g);
-int parse_config_profile (FILE *config, profile *p, const unsigned int has_sysfs);
+int parse_config_profile (FILE *config, profile *p, general *g);
 int parse_config_rule (FILE *config, rule *r);
 int parse_config_general (FILE *config, general *g);
 void free_config(general *g);
