@@ -153,6 +153,7 @@ int main (int argc, char *argv[]) {
   
   /* free configuration */
   free_config(configuration);
+  free(configuration);
   
   /* TODO: this call to close log shound not sit here... */
   closelog();
@@ -423,6 +424,7 @@ int main_loop(void) {
       sleep(configuration->poll_interval);
   } /* end loop */
 
+  free(_system);
   return 0;
 }
 
@@ -506,7 +508,7 @@ int clear_cpufreqd_pid(void) {
  */
 int set_policy(profile *prof) {
   FILE *fp;
-  char *path = (char *)malloc(256);
+  char path[256];
   
   if (!prof)
     return -1;
@@ -852,7 +854,7 @@ int read_args (int argc, char *argv[]) {
     { "version", 0, 0, 'v' }, 
     { "file", 1, 0, 'f' },
     { "no-daemon", 0, 0, 'D' },
-    { "verbosity", 0, 0, 'V' },
+    { "verbosity", 1, 0, 'V' },
     { 0, 0, 0, 0 }, 
   };
   int ch,option_index = 0;
